# coding: utf-8
from django.contrib.auth.decorators import login_required
from django.shortcuts import get_object_or_404, redirect, render
from django.utils.timezone import now
from .forms import FonteExternaForm
from .models import FonteExterna
from .sincronizador import Sincronizador


@login_required
def home_fonte_externa(request):
    fontes_recentes = FonteExterna.objects.all()[:5].select_related()
    return render(request, "onivoro/home.html", {"fontes": fontes_recentes})


@login_required
def criar_fonte_externa(request):
    form = FonteExternaForm(request.POST or None)

    if form.is_valid():

        externa = form.save(commit=False)
        externa.criado_em = now()
        externa.atualizado_em = now()
        externa.save()

        return redirect("detalhe_fonte_externa", id=externa.id)

    else:
        return render(request, "onivoro/criar.html", {"form": form})


@login_required
def detalhe_fonte_externa(request, id):
    fonte = get_object_or_404(FonteExterna, id=id)
    dados_importados = fonte.ref_modelo.objects.all()[:5]

    contador_importados = fonte.ref_modelo.objects.all().count()

    # camada = CamadaGeoJSON.objects.get(app=fonte.app, modelo=fonte.modelo)
    camada = "" # TODO resolver isto

    dicionario = {"fonte": fonte,
                  "camada": camada,
                  "dados_importados": dados_importados,
                  "contador_importacao": contador_importados}

    return render(request, "onivoro/detalhe.html", dicionario)


@login_required
def editar_fonte_externa(request, id):
    instancia = get_object_or_404(FonteExterna, pk=id)
    form = FonteExternaForm(request.POST or None, instance=instancia)

    if form.is_valid():

        externa = form.save(commit=False)
        externa.criado_em = now()
        externa.atualizado_em = now()
        externa.save()
        return redirect("detalhe_fonte_externa", id=externa.id)

    else:
        return render(request, "onivoro/editar.html", {"form": form})


@login_required
def excluir_fonte_externa(request, id):
    instancia = get_object_or_404(FonteExterna, pk=id)

    if request.POST:

        instancia.delete()
        return render(request, "onivoro/excluir.html", {})
    else:

        return redirect("home_fonte_externa")


@login_required
def sincronizar_fonte_externa(request, id):
    fonte = get_object_or_404(FonteExterna, pk=id)

    sync = Sincronizador(fonte)
    sucesso = sync.sincronizar()

    return render(request, "onivoro/sincronizado.html", {"fonte_externa": fonte, "sucesso": sucesso})


@login_required
def lista_fonte_externa(request):
    pass


# -*- coding: utf-8 -*-
# Generated by Django 1.9.6 on 2016-05-17 23:13
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='FonteExterna',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('app', models.CharField(help_text='Aplica\xe7\xe3o Django que cont\xe9m o modelo em quest\xe3o', max_length=128, verbose_name='Aplica\xe7\xe3o Django')),
                ('modelo', models.CharField(help_text='Modelo Django que ser\xe1 utilizado.', max_length=128, verbose_name='Modelo Django')),
                ('criado_em', models.DateTimeField(auto_now_add=True, help_text='Data de cria\xe7\xe3o da fonte externa.', verbose_name='Criado em')),
                ('atualizado_em', models.DateTimeField(auto_now=True, help_text='Data da \xfaltima atualiza\xe7\xe3o desta fonte externa.', verbose_name='Atualizado em')),
                ('nome', models.CharField(help_text='Texto identificador da fonte externa.', max_length=128, unique=True, verbose_name='Nome')),
                ('descricao', models.CharField(help_text='Texto descritivo da fonte externa', max_length=512, verbose_name='Descri\xe7\xe3o da Fonte Externa')),
                ('srid', models.IntegerField(default=4326, help_text='SRID do sistema de refer\xeancia de entrada.', verbose_name='SRID (nativo)')),
                ('encoding', models.CharField(default=b'latin-1', help_text='Codifica\xe7\xe3o em que o dado original se encontra. Ex: utf-8, latin-1, etc. Padr\xe3o \xe9 latin-1.', max_length=16, verbose_name='Codifica\xe7\xe3o')),
                ('tipo', models.CharField(choices=[('SHAPEFILE', 'Shapefile'), ('GEOJSON', 'GeoJson'), ('WFS', 'WFS (Web Feature Service)')], max_length=32, verbose_name='Tipo da Fonte Externa')),
                ('url_sincronia', models.URLField(help_text='URL do recurso a ser sincronizado', verbose_name='Endere\xe7o Sincronia')),
                ('formato', models.CharField(choices=[('SHP', 'Shapefile'), ('JSON', 'GeoJson'), ('ZIP', 'Zip'), ('TAR.GZ', 'Tarball'), ('REQUEST', 'Request HTTP')], help_text='Formato do arquivo utilizado para sincronzia\xe7\xe3o.', max_length=6, verbose_name='Formato Arquivo')),
                ('ultima_sincronizacao', models.DateTimeField(help_text='Data em que o modelo foi sincronizado pela \xfaltima vez.', verbose_name='\xdaltima sincroniza\xe7\xe3o')),
                ('nome_alvo', models.CharField(blank=True, help_text='Nome do arquivo alvo. S\xf3 \xe9 necess\xe1rio se um zip cont\xe9m mais de um arquivo do mesmo tipo.', max_length=128, null=True, verbose_name='Nome do Arquivo Alvo')),
            ],
            options={
                'ordering': ('-atualizado_em',),
                'verbose_name': 'Fonte Externa',
                'verbose_name_plural': 'Fontes Externas',
            },
        ),
        migrations.CreateModel(
            name='Mapeamento',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('campo_origem', models.CharField(help_text='Nome do campo, na origem (fonte externa).', max_length=64, verbose_name='Nome do Campo (fonte externa)')),
                ('campo_destino', models.CharField(help_text='Nome do campo, no destino (modelo GeoDjango).', max_length=64, verbose_name='Nome do Campo (modelo GeoDjango)')),
                ('campo_destino_estrangeira', models.CharField(blank=True, help_text='Chave estrangeira do modelo.', max_length=64, null=True, verbose_name='Nome do Campo Chave Estrangeira (modelo GeoDjango)')),
                ('tipo_origem', models.CharField(choices=[(b'GEO', 'Geometria'), (b'INT', 'Inteiro'), (b'FLOAT', 'Decimal'), (b'STRING', 'Texto'), (b'DATA', 'Data'), (b'DATA-HORA', 'Data/Hora'), (b'FK', 'Chave Estrangeira')], editable=False, help_text='Tipo do campo, na origem (fonte externa).', max_length=32, verbose_name='Tipo do Campo (fonte externa)')),
                ('tipo_destino', models.CharField(choices=[(b'GEO', 'Geometria'), (b'INT', 'Inteiro'), (b'FLOAT', 'Decimal'), (b'STRING', 'Texto'), (b'DATA', 'Data'), (b'DATA-HORA', 'Data/Hora'), (b'FK', 'Chave Estrangeira')], editable=False, help_text='Tipo do Campo, no destino (modelo GeoDjango).', max_length=32, verbose_name='Tipo do Campo (modelo GeoDjango)')),
                ('fonte_externa', models.ForeignKey(help_text='Fonte externa em que este mapeamento de campos est\xe1 associado.', on_delete=django.db.models.deletion.CASCADE, related_name='mapeamentos', to='onivoro.FonteExterna', verbose_name='Fonte Externa')),
            ],
            options={
                'verbose_name': 'Mapeamento de Campo',
                'verbose_name_plural': 'Mapeamento de Campos',
            },
        ),
        migrations.CreateModel(
            name='Transformacao',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('nome', models.CharField(help_text='Nome da Transforma\xe7\xe3o', max_length=256, unique=True, verbose_name='Nome')),
                ('descricao', models.CharField(help_text='Descri\xe7\xe3o da Transforma\xe7\xe3o.', max_length=256, verbose_name='Descri\xe7\xe3o')),
                ('tipo_campo', models.CharField(choices=[(b'GEO', 'Geometria'), (b'INT', 'Inteiro'), (b'FLOAT', 'Decimal'), (b'STRING', 'Texto'), (b'DATA', 'Data'), (b'DATA-HORA', 'Data/Hora'), (b'FK', 'Chave Estrangeira')], help_text='Tipo de Campo a qual esta transforma\xe7\xe3o se aplica.', max_length=32, verbose_name='Tipo de Campo Aplic\xe1vel')),
                ('metodo_conversao', models.TextField(help_text='C\xf3digo que ser\xe1 utilizado para transformar os valores. A entrada dever\xe1 ser obrigat\xf3riamente uma fun\xe7\xe3o com um argumento e exatamente um retorno.', verbose_name='C\xf3digo utilizado na transforma\xe7\xe3o')),
            ],
            options={
                'verbose_name': 'Transforma\xe7\xe3o',
                'verbose_name_plural': 'Transforma\xe7\xf5es',
            },
        ),
        migrations.AddField(
            model_name='mapeamento',
            name='transformacoes',
            field=models.ManyToManyField(help_text='Transforma\xe7\xf5es ser\xe3o executadas para cada campo, de forma a adequar o mesmo ao modelo interno.', to='onivoro.Transformacao', verbose_name='Transforma\xe7\xf5es'),
        ),
        migrations.AlterUniqueTogether(
            name='mapeamento',
            unique_together=set([('fonte_externa', 'campo_destino')]),
        ),
    ]



# coding: utf-8


class SincronizadorException(Exception):
    pass


class TransformacaoException(Exception):
    """
    Exceção lançada por uma transformação do dge.
    """
    pass


class CodigoPerigosoException(TransformacaoException):
    """
    Exceção lançada quando código potencialmente perigoso é detectado.
    """
    pass


class MapeamentoException(Exception):
    pass



# coding: utf-8
from django.utils.translation import ugettext_lazy as _

EXTERNAL_SOURCE_TYPE = (
    (u"SHAPEFILE",_(u"Shapefile")),
    (u"GEOJSON",_(u"GeoJson")),
    (u"WFS",_(u"WFS (Web Feature Service)")),
)

EXTERNAL_SOURCE_FORMAT = (
    (u"SHP",_(u"Shapefile")),
    (u"JSON", _(u"GeoJson")),
    (u"ZIP", _(u"Zip")),
    (u"TAR.GZ", _(u"Tarball")),
    (u"REQUEST", _(u"Request HTTP"))
)

FIELD_TYPE_GEO = "GEO"
FIELD_TYPE_INT = "INT"
FIELD_TYPE_FLOAT = "FLOAT"
FIELD_TYPE_STRING = "STRING"
FIELD_TYPE_DATA = "DATA"
FIELD_TYPE_DATA_HORA = "DATA-HORA"
FIELD_TYPE_FK = "FK"

FIELD_TYPE = (
    (FIELD_TYPE_GEO, _(u"Geometria")),
    (FIELD_TYPE_INT, _(u"Inteiro")),
    (FIELD_TYPE_FLOAT, _(u"Decimal")),
    (FIELD_TYPE_STRING, _(u"Texto")),
    (FIELD_TYPE_DATA, _(u"Data")),
    (FIELD_TYPE_DATA_HORA, _(u"Data/Hora")),
    (FIELD_TYPE_FK, _(u"Chave Estrangeira"))
)

FIELD_MAPPING_ERROR_TYPE = ((_(u"MISSING_FIELD"), _(u"Campo Não Existe")),
                            (_(u"INCOMPATIBLE_TYPE"), _(u"Tipo Incompatível")),
                            (_(u"INCOMPATIBLE_SIZE"), _(u"Tamanho Incompatível"))
)



# coding: utf-8
import logging
from custom_settings.loader import load_settings


load_settings(__name__)

# configuring no-op handler for our logger. users should add specific handlers for this.
log = logging.getLogger("django-onivoro")
log.addHandler(logging.NullHandler())

__version__ = '0.1.0'



# coding: utf-8
from django import forms
from .models import FonteExterna


class FonteExternaForm(forms.ModelForm):
    class Meta:
        model = FonteExterna
        fields = ('nome',
                  'tipo',
                  'url_sincronia',
                  'formato',
                  'descricao',
                  'srid',
                  'encoding', )


# coding: utf-8
from django.conf.urls import patterns, url
from .views import (
    home_fonte_externa,
    criar_fonte_externa,
    detalhe_fonte_externa,
    editar_fonte_externa,
    excluir_fonte_externa,
    sincronizar_fonte_externa,
)

urlpatterns = patterns(
    "",
    url(r"^$", home_fonte_externa, name="home_fonte_externa"),
    url(r"^criar/$", criar_fonte_externa, name="criar_fonte_externa"),
    url(r"^detalhe/(?P<id>\d+)/$", detalhe_fonte_externa, name="detalhe_fonte_externa"),
    url(r"^editar/(?P<id>\d+)/$", editar_fonte_externa, name="editar_fonte_externa"),
    url(r"^excluir/(?P<id>\d+)/$", excluir_fonte_externa, name="excluir_fonte_externa"),
    url(r"^sincronizar/(?P<id>\d+)/$", sincronizar_fonte_externa, name="sincronizar_fonte_externa"),
)



# -*- coding: utf-8 -*-
import compiler
import logging
try:
    from django.db.models import get_model
except:
    from django.apps import apps
    get_model = apps.get_model
from django.utils.translation import ugettext_lazy as _
from django.contrib.gis.db import models
from .choices import (
    EXTERNAL_SOURCE_TYPE,
    EXTERNAL_SOURCE_FORMAT,
    FIELD_TYPE,
    FIELD_MAPPING_ERROR_TYPE,
)
from .exceptions import (
    TransformacaoException,
    CodigoPerigosoException,
)
log = logging.getLogger("django-onivoro")


class AppModelMixin(models.Model):

    """
    Este modelo tem em seus atributos dados para referenciar uma
    aplicação e um modelo django.
    """

    app = models.CharField(max_length=128,
                           verbose_name=_(u"Aplicação Django"),
                           help_text=_(u"Aplicação Django que contém o modelo em questão"))

    modelo = models.CharField(max_length=128,
                              verbose_name=_(u"Modelo Django"),
                              help_text=_(u"Modelo Django que será utilizado."))

    @property
    def validar_modelo(self):
        """
        Valida se o modelo é válido ou não.
        """
        try:
            modelo = get_model(self.app, self.modelo)
            if modelo is None:
                return False
            else:
                return True
        except Exception:
            # todo: qualificar a exceção que é lançada quando não achamos um modelo.
            return False

    @property
    def ref_modelo(self):
        """
        Retorna uma referência ao modelo Django.
        Caso o modelo não exista ou seja inválido, retorna None.
        todo: este método duplica uma chamada a get_model. Reduzir esta duplicação.
        """
        if self.validar_modelo:
            return get_model(self.app, self.modelo)
        else:
            return None

    class Meta:
        abstract = True


class FonteExterna(AppModelMixin):
    """
    Classe utilizada para representar uma fonte externa, capaz de ser atualizada
    através de comandos de gerenciamento.
    """

    criado_em = models.DateTimeField(auto_now_add=True,
                                     editable=False,
                                     verbose_name=_(u"Criado em"),
                                     help_text=_(u"Data de criação da fonte externa."))

    atualizado_em = models.DateTimeField(auto_now=True,
                                         editable=False,
                                         verbose_name=_(u"Atualizado em"),
                                         help_text=_(u"Data da última atualização desta fonte externa."))

    nome = models.CharField(max_length=128,
                            unique=True,
                            verbose_name=_(u"Nome"),
                            help_text=_(u"Texto identificador da fonte externa."))

    descricao = models.CharField(max_length=512,
                                 verbose_name=_(u"Descrição da Fonte Externa"),
                                 help_text=_(u"Texto descritivo da fonte externa"))

    srid = models.IntegerField(verbose_name=_(u"SRID (nativo)"), 
                               help_text=_(u"SRID do sistema de referência de entrada."),
                               default=4326)

    encoding = models.CharField(max_length=16,
                                verbose_name=_(u"Codificação"),
                                help_text=_(u"Codificação em que o dado original se encontra. Ex: utf-8, latin-1, etc. Padrão é latin-1."),
                                default="latin-1")

    tipo = models.CharField(max_length=32,
                            verbose_name=_(u"Tipo da Fonte Externa"),
                            choices=EXTERNAL_SOURCE_TYPE)

    url_sincronia = models.URLField(verbose_name=_(u"Endereço Sincronia"),
                                    help_text=_(u"URL do recurso a ser sincronizado"))

    formato = models.CharField(max_length=6,
                               choices=EXTERNAL_SOURCE_FORMAT,
                               verbose_name=_(u"Formato Arquivo"),
                               help_text=_(u"Formato do arquivo utilizado para sincronziação."))

    ultima_sincronizacao = models.DateTimeField(auto_now=False,
                                                verbose_name=_(u"Última sincronização"),
                                                help_text=_(u"Data em que o modelo foi sincronizado pela última vez."))

    nome_alvo = models.CharField(max_length=128,
                                 verbose_name=_(u"Nome do Arquivo Alvo"),
                                 help_text=_(u"Nome do arquivo alvo. Só é necessário se um zip contém mais de um arquivo do mesmo tipo."),
                                 null=True,
                                 blank=True)

    @property
    def mapa_campos(self):
        """
        Retorna um dicionário com o mapa de campos
        """
        mapa = {}

        for fm in self.mapeamentos.all():
            if mapa.has_key(fm.campo_destino):
                raise MapeamentoException(
                    _(u"Não é possível ter duas chaves com o mesmo valor. Não é possível criar este mapeamento de campos."))

            if fm.tipo_destino == "FK":
                mapa[fm.campo_destino] = {fm.campo_destino_estrangeira: fm.campo_origem}
            else:
                mapa[fm.campo_destino] = fm.campo_origem

        return mapa

    @property
    def deve_limpar_registros_antigos(self):
        """
        Esta propriedade diz se devemos limpar os
        dados antigos apos a importaçao dos
        novos dados.

        Basicamente verificamos se a PK possui
        nome id. Se verdadeiro, devemos executar
        a limpeza, pois não podemos confiar no
        comportamento de LayerMapping (onde existe PK
        confiável, ele atualiza os dados e faz inserts
        onde necessário).
        """
        if self.ref_modelo._meta.pk.name == "id":
            return True

        return False

    def url_valida(self):
        """
        Valida a URL com o tipo.
        """

        extensao = self.url_sincronia[-3:].upper()

        if self.tipo == "SHAPEFILE":
            if extensao not in ("ZIP", "SHP") and self.url_sincronia[-6:].upper() != "TAR.GZ":
                return False

            return True
        elif self.tipo == "GEOJSON":
            if extensao not in ("", "ZIP", "TAR.GZ", "JSON") and not extensao.endswith("/"):
                return False

            return True

        elif self.tipo == "WFS":
            if extensao != "":
                return False

            return True
        else:
            raise Exception(_("Tipo de fonte externa não suportado."))

    def __unicode__(self):
        """
        Representação unicode do modelo FonteExterna
        """
        return unicode(self.nome)

    def get_absolute_url(self):
        """
        Stub de permalink para FonteExterna
        todo: implementar permalink para fonte externa
        """
        return ("detalhe_fonte_externa", [str(self.id)])

    def save(self, *args, **kwargs):
        """
        Método customizado de salvar o objeto.
        """
        if not self.validar_modelo:
            raise ValueError(_(u"Não foi possível carregar o modelo especificado."))

        super(FonteExterna, self).save(*args, **kwargs)

    class Meta:
        verbose_name = _(u"Fonte Externa")
        verbose_name_plural = _(u"Fontes Externas")
        ordering = ("-atualizado_em",)


class Transformacao(models.Model):
    """
    Modelo que transforma valores de um lado para outro no ETL.
    Transformações possíveis são arredondamentos, encurtamentos de string,
    tranformações de SRID, etc.
    Este modelo é extremamente sensível, portanto somente pessoas de extrema
    confiança devem atualizá-lo.
    """

    nome = models.CharField(max_length=256,
                            verbose_name=_(u"Nome"),
                            help_text=_(u"Nome da Transformação"),
                            unique=True)

    # este campo é obrigatório para facilitar a identificação das transformações.
    descricao = models.CharField(max_length=256,
                                 verbose_name=_(u"Descrição"),
                                 help_text=_(u"Descrição da Transformação."))

    tipo_campo = models.CharField(max_length=32,
                                  choices=FIELD_TYPE,
                                  verbose_name=_(u"Tipo de Campo Aplicável"),
                                  help_text=_(u"Tipo de Campo a qual esta transformação se aplica."))

    metodo_conversao = models.TextField(verbose_name=_(u"Código utilizado na transformação"),
                                        help_text=_(u"Código que será utilizado para transformar os valores. A entrada deverá ser obrigatóriamente uma função com um argumento e exatamente um retorno."))

    _metodo_conversao = None

    def __unicode__(self):
        return self.nome

    def get_absolute_url(self):
        """
        Stub de permalink para Transformação
        todo: implementar get_absolute_url em Transformacao
        """
        pass

    def codigo_perigoso(self):
        """
        Método que tenta identificar se um código é perigoso
        ou não.
        Sei que esta tentativa é bem simples, mas a idéia é que poucos usuários
        tenham acesso a este modelo.
        """
        # exec
        if self.metodo_conversao.find("exec") != -1:
            log.info(_("Contém um statement exec. Código potencialmente perigoso."))
            return True

        # eval
        if self.metodo_conversao.find("eval") != -1:
            log.info(_("Contém um statement eval. Código potencialmente perigoso."))
            return True

        # import
        if self.metodo_conversao.find("import") != -1:
            log.info(_("Contém um statement import. Código potencialmente perigoso."))
            return True

        # import
        if self.metodo_conversao.find("from") != -1:
            log.info(_("Contém um statement from. Código potencialmente perigoso."))
            return True

        # wild import
        if self.metodo_conversao.find("*") != -1:
            log.info(_("Contém um statement * 'all'. Código potencialmente perigoso."))
            return True

        # whiles não são permitidos
        if self.metodo_conversao.find("while") != -1:
            log.info(_("Contém um statement while. Código potencialmente perigoso."))
            return True

        return False

    def sintaxe_valida(self):
        """
        Testa se a sintaxe do código é válida e se
        o código contém, por obrigatoriedade uma função
        chamada converter, com dois parametros (entrada e campo_novo),
        retornando um valor especifico.
        Exemplo:

        def converter(entrada,campo_novo):
            return "a"
        """

        try:
            mod = compiler.parse(self.metodo_conversao)

            # em um mundo ideal, sem imports, sem outras funcoes, etc

            #            funcao_converte = mod.getChildren()[1].getChildren()[0]
            #
            #            if funcao_converte.name != "converte":
            #                log.info(u"O código não atende as especificações (primeira função do módulo deve se chamar 'converte').")
            #                return False
            #
            #            if len(funcao_converte.argnames) != 2:
            #                log.info(u"O código não atende as especificações (a função converte deve ter somente dois argumentos).")
            #                return False
            #
            #            if funcao_converte.argnames[0] != "entrada":
            #                log.info(u"O código não atende as especificações (primeiro parametro da função converte deve se chamar entrada).")
            #                return False
            #
            #            if funcao_converte.argnames[0] != "campo_novo":
            #                log.info(u"O código não atende as especificações (segundo parametro da função converte deve se chamar campo_novo).")
            #                return False

            return True
        except SyntaxError as synEx:
            return False

    def transforma(self, entrada, campo_novo):
        """
        Método para transformação de valores
        """

        log.info(_("Transformando valor via %s" % self.nome))

        if not self.sintaxe_valida():
            raise TransformacaoException(_(u"A sintaxe do código não é válida."))

        if self.codigo_perigoso():
            log.info(_("Código considerado perigoso. Não será executado."))
            raise ValueError(
                _(u"O código da transformação foi considerado potencialmente perigoso e não poderá ser salvo/executado no sistema."))

        valor_novo = None

        try:
            if not self._metodo_conversao:
                self._metodo_conversao = compile(self.metodo_conversao, "<string>", "exec")

            ns = {"__builtins__": __builtins__, "entrada": entrada, "campo_novo": campo_novo}
            exec self._metodo_conversao in ns

            return ns["valor_novo"]

        except Exception as ex:
            return None

    def save(self, *args, **kwargs):
        """
        Método customizado de save. Impede que código potencialmente perigoso seja
        armazenado e ou executado pelo servidor.
        """

        if not self.sintaxe_valida():
            raise TransformacaoException(_(u"A sintaxe do código não é válida."))

        if self.codigo_perigoso():
            raise CodigoPerigosoException(
                _(u"O código da transformação foi considerado potencialmente perigoso e não poderá ser salvo/executado no sistema."))

        super(Transformacao, self).save(*args, **kwargs)

    class Meta:
        verbose_name = _(u"Transformação")
        verbose_name_plural = _(u"Transformações")


class Mapeamento(models.Model):
    """
    Mapeamento entre dois campos, um externo e um interno.
    """

    fonte_externa = models.ForeignKey(FonteExterna,
                                      verbose_name=_(u"Fonte Externa"),
                                      help_text=_(u"Fonte externa em que este mapeamento de campos está associado."),
                                      related_name="mapeamentos")

    campo_origem = models.CharField(max_length=64,
                                    verbose_name=_(u"Nome do Campo (fonte externa)"),
                                    help_text=_(u"Nome do campo, na origem (fonte externa)."))

    campo_destino = models.CharField(max_length=64,
                                     verbose_name=_(u"Nome do Campo (modelo GeoDjango)"),
                                     help_text=_(u"Nome do campo, no destino (modelo GeoDjango)."))

    campo_destino_estrangeira = models.CharField(max_length=64,
                                                 null=True,
                                                 blank=True,
                                                 verbose_name=_(u"Nome do Campo Chave Estrangeira (modelo GeoDjango)"),
                                                 help_text=_(u"Chave estrangeira do modelo."))

    tipo_origem = models.CharField(max_length=32,
                                   choices=FIELD_TYPE,
                                   editable=False,
                                   verbose_name=_(u"Tipo do Campo (fonte externa)"),
                                   help_text=_(u"Tipo do campo, na origem (fonte externa)."))

    tipo_destino = models.CharField(max_length=32,
                                    choices=FIELD_TYPE,
                                    editable=False,
                                    verbose_name=_(u"Tipo do Campo (modelo GeoDjango)"),
                                    help_text=_(u"Tipo do Campo, no destino (modelo GeoDjango)."))

    transformacoes = models.ManyToManyField(Transformacao,
                                            verbose_name=_(u"Transformações"),
                                            help_text=_(u"Transformações serão executadas para cada campo, de forma a adequar o mesmo ao modelo interno."))

    def transforma(self, registro_origem, registro_destino):
        pass

    def conversao_valida(self):
        """
        Valida se o mapeamento de campos é coerente e está tudo no lugar.
        """

        if self.tipo_origem == self.tipo_destino:
            # ok, o tipo é o mesmo.
            return True

        if self.tipo_origem == "STRING" and self.tipo_destino != "STRING" and self.tipo_destino != "FK":
            # o tipo de string é complicado de mapear para outros tipos, portanto vamos desabilitar
            return False

        if self.tipo_origem == "GEO" and self.tipo_destino != "GEO":
            # por enquanto só vamos converter geom para geom
            return False

        if self.tipo_origem in ("INT", "FLOAT", "DATA", "DATA-HORA") and self.tipo_destino == "STRING":
            # ok, podemos fazer uma conversão coxa
            return True

        if self.tipo_origem in ("INT", "FLOAT") and self.tipo_destino in ("INT", "FLOAT"):
            # ok, podemos truncar o valor ou subir a conversão
            return True

        if self.tipo_origem in ("DATA", "DATA-HORA") and self.tipo_destino == ("DATA", "DATA-HORA"):
            # ok, podemos passar data sem horas ou truncar a hora
            return True

        if self.tipo_destino == "FK" and self.campo_destino_estrangeira:
            # se o tipo for FK, é necessário especificar quem é destino final da bagaça!
            return True

        return False

    def __unicode__(self):
        """
        Representação de um mapeamento.
        Exemplo:
        [Unidades de Conservação] nome_uc -> nome_oficial_uc
        """
        modelo = get_model(self.fonte_externa.app, self.fonte_externa.modelo)

        return "[%s] - %s -> %s" % (modelo._meta.verbose_name, self.campo_origem, self.campo_destino)

    class Meta:
        # não é possível ter dois campos apontando para somente um ou um apontando para dois.
        unique_together = (("fonte_externa", "campo_destino"),)

        verbose_name = _(u"Mapeamento de Campo")
        verbose_name_plural = _(u"Mapeamento de Campos")



# coding: utf-8
from django.contrib.gis.gdal.field import *

__author__ = 'George'

class OGRField(object):
    """
    Classe para representar um campo OGR (que vem de um Layer do DataSource)
    """

    name = None
    type = None
    width = None
    precision = None

    def __init__(self,name,type,width,precision):
        self.name = name
        self.type = type
        self.width = width
        self.precision = precision

    def __unicode__(self):
        return self.name

    def __str__(self):
        return self.name

class ErroMapeamento(object):
    """
    Classe simples para segurar um erro de mapeamento.
    """

    mapeamento = None
    mensagem = None
    tipo_erro = None

    def __init__(self,mapeamento,tipo_erro,mensagem):
        self.mapeamento = mapeamento
        self.tipo_erro = tipo_erro
        self.mensagem = mensagem

    def __unicode__(self):
        return "%s - %s" % (self.tipo_erro,self.mensagem)

class ConversorTipoCampo(object):
    """
    Esta classe compatibiliza os tipos de campos
    encontrados no OGR com o nosso mapeamento.
    """

    def __init__(self):
        pass

    @staticmethod
    def tipo_coincide(campo_ogr,campo_mapeamento,lado="DESTINO"):
        """
        Compara os tipos de dois campos ogr e mapeados
        e veem se sao compativeis.
        Caso lado seja diferente de "DESTINO", ele irá retornar o campo da origem.
        Como esta verificação é mais para os campos externos que podem mudar mais,
        faz mais sentido deixar este o default.
        """

        if lado == "DESTINO":
            return ConversorTipoCampo._tipo_coincide_impl(campo_ogr.type,campo_mapeamento.tipo_destino)
        else:
            return ConversorTipoCampo._tipo_coincide_impl(campo_ogr.type,campo_mapeamento.tipo_origem)

    @staticmethod
    def tipo_coincide(tipo_ogr,tipo_mapeamento):
        """
        Compara dois tipos de campos e veem se sao compativeis
        """
        return ConversorTipoCampo._tipo_coincide_impl(tipo_ogr,tipo_mapeamento)

    @staticmethod
    def _tipo_coincide_impl(ogr_code,campo_mapeamento):
        """
        Implementação para comparacao de tipos
        """

        if ogr_code in (OFTIntegerList,OFTRealList,OFTStringList,OFTWideStringList,OFTTime,OFTBinary):
            # não temos equivalentes aos campos lista.
            return False

        if ogr_code == OFTInteger and campo_mapeamento == "INT":
            return True

        if ogr_code == OFTReal and campo_mapeamento == "FLOAT":
            return True

        if ogr_code == OFTString and campo_mapeamento == "STRING":
            return True

        if ogr_code == OFTWideString and campo_mapeamento == "STRING":
            return True

        if ogr_code == OFTDate and campo_mapeamento == "DATA":
            return True

        if ogr_code == OFTDateTime and campo_mapeamento == "DATA-HORA":
            return True

        if campo_mapeamento == "FK":
            # não temos como verificar se esta FK coincide com o que precisamos.
            return True

        return False

    @staticmethod
    def mapeamento_para_ogr(campo_mapeamento):
        pass

    @staticmethod
    def ogr_para_mapeamento(campo_ogr):
        pass


# coding: utf-8
import tarfile
import shutil
import os
import urllib2
import logging
from zipfile import ZipFile
from django.utils.translation import ugettext_lazy as _
from django.utils.timezone import now
from django.template.loader import get_template
from django.template import Context
from django.core.mail import send_mail
from django.conf import settings
from django.contrib.gis.utils.layermapping import LayerMapping
from django.db.models.query_utils import Q
from django.contrib.gis.gdal import DataSource
from .signals import (
    pre_sync,
    post_sync
)
from .exceptions import SincronizadorException
from .ogr_utils import OGRField, ErroMapeamento, ConversorTipoCampo


log = logging.getLogger("django-onivoro")


def notificar(sucesso, fonte_externa, count_anterior=None, count_atual=None, mensagem=None):
    if not settings.ONIVORO_MAIL_NOTIFY:
        return

    if not settings.ONIVORO_MAIL_RECIPIENTS:
        return

    if not settings.ONIVORO_MAIL_SENDER:
        return

    if sucesso:
        assunto = _(u"[Django-Onivoro] Sucesso")
        template = get_template("onivoro/email_sucesso.txt")
    else:
        assunto = _(u"[Django-Onivoro] Falha")
        template = get_template("onivoro/email_falha.txt")

    context = Context({"fonte_externa": fonte_externa,
                       "registros_atual": count_atual,
                       "registros_anterior": count_anterior,
                       "mensagem": mensagem,
                       "data": now()})

    email = template.render(context)

    send_mail(assunto, email, settings.ONIVORO_MAIL_SENDER, settings.ONIVORO_MAIL_RECIPIENTS, fail_silently=True)


class Sincronizador(object):
    """
    Classe que sincroniza entre uma fonte externa e um modelo geo.
    Em tese esta classe suporta todo um range de formatos, listados em:
    http://www.gdal.org/ogr/ogr_formats.html
    """

    # fonte externa a ser sincronizada
    fonte_externa = None

    # arquivo mais antigo importado
    arquivo_v2 = None

    # ultimo arquivo importado
    arquivo_v1 = None

    # arquivo sendo importado agora
    arquivo_v0 = None

    # caminho para o arquivo de fonte que será lido e importado (.json, .shp, etc)
    arquivo_alvo = None

    def __init__(self, fonte_externa):

        if not os.path.exists(settings.ONIVORO_DATA_ROOT):
            os.makedirs(settings.ONIVORO_DATA_ROOT)

        self.fonte_externa = fonte_externa

    def sincronizar(self):
        """
        Principal método do sincronizador.
        """
        pre_sync.send_robust(sender=self, fonte_externa=self.fonte_externa)
        sucesso = False
        numero_registros_anterior = None
        try:

            numero_registros_anterior = self.fonte_externa.ref_modelo.objects.all().count()

            if (self.fonte_externa.deve_limpar_registros_antigos and
               self.fonte_externa.ref_modelo.objects.all().count() > 0):

                max_id = self.fonte_externa.ref_modelo.objects.order_by("-id")[0].id
            else:
                max_id = -1

            self._download()

            self._carrega()

            sucesso = True

            # se existiam registros e o número de registros novos for > 0, remova os antigos
            if max_id > 0 and self.fonte_externa.ref_modelo.objects.order_by("-id")[0].id - max_id != 0:
                self.fonte_externa.ref_modelo.objects.filter(id__lte=max_id).delete()

            numero_registros_atual = self.fonte_externa.ref_modelo.objects.all().count()

            notificar(sucesso,
                      self.fonte_externa,
                      numero_registros_anterior,
                      numero_registros_atual,
                      "Sucesso!")
            post_sync.send_robust(sender=self, fonte_externa=self.fonte_externa, sucesso=True)

        except Exception as exc:
            notificar(sucesso,
                      self.fonte_externa,
                      None,
                      numero_registros_anterior,
                      exc.message)
            post_sync.send_robust(sender=self, fonte_externa=self.fonte_externa, sucesso=False)
            log.error(exc.message)

    def _download(self, url=""):

        if url:
            self.fonte_externa.url_sincronia = url

        try:

            request = urllib2.Request(self.fonte_externa.url_sincronia)
            remoto = urllib2.urlopen(request)
            nome = os.path.split(self.fonte_externa.url_sincronia)[-1]

            # remove o arquivo mais antigo
            self.arquivo_v2 = os.path.join(settings.ONIVORO_DATA_ROOT, nome + ".old")
            if os.path.exists(self.arquivo_v2):
                os.remove(os.path.join(settings.ONIVORO_DATA_ROOT, nome + ".old"))

            # renomeia o antigo para .old
            self.arquivo_v1 = os.path.join(settings.ONIVORO_DATA_ROOT, nome)
            if os.path.exists(self.arquivo_v1):
                os.rename(os.path.join(settings.ONIVORO_DATA_ROOT, nome),os.path.join(settings.ONIVORO_DATA_ROOT, nome + ".old"))

            if self.fonte_externa.formato in ("TAR.GZ","ZIP","SHP","SHX","DBF"):
                formato = "wb"
            elif self.fonte_externa.formato == "REQUEST":
                formato = "w"
            else:
                raise SincronizadorException(u"Não foi possível determinar o formato dos dados para sincronização.")

            self.arquivo_v0 = os.path.join(settings.ONIVORO_DATA_ROOT, nome)
            local = open(self.arquivo_v0, formato)
            local.write(remoto.read())
            local.close()

            if self.fonte_externa.tipo == "SHAPEFILE" and nome[-3:].upper() == "SHP":
                # caso esta fonte externa seja um shapefile puro, ele também é o arquivo alvo!
                self.arquivo_alvo = self.arquivo_v0

                # caso esta fonte externa seja um shapefile puro, devemos baixar também seus complementos, dbf, shx e prj (opcional)
                dbf = self.fonte_externa.url_sincronia.replace("shp", "dbf")
                shx = self.fonte_externa.url_sincronia.replace("shp", "shx")

                self._download(url=dbf)
                self._download(url=shx)

            if nome[-3:].upper() in ("ZIP", "RAR") or nome[-6:].upper == "TAR.GZ":
                self._descompactar()

        except urllib2.HTTPError as httpEx:

            mensagem = u"Ocorreu um problema HTTP.\n%s" % httpEx.message
            log.error(mensagem)
            raise

        except urllib2.URLError as urlEx:

            mensagem = u"Ocorreu um problema de URL.\n %s" % urlEx.message
            log.error(mensagem)
            raise

    def _descompactar_zip(self):
        """
        Descompacta um arquivo zip e retorna o arquivo
        principal de fonte de dados.
        """
        nome_arquivo = os.path.split(self.arquivo_v0)[-1]
        zip = ZipFile(self.arquivo_v0)

        badzips = zip.testzip()

        if badzips:
            raise SincronizadorException("Não é possível sincronizar a fonte externa %s pois o arquivo zip %s está corrompido ou danificado. Arquivo %s com problemas." % (self.fonte_externa,self.arquivo_v0,badzips))

        unzip_dir = os.path.join(os.path.dirname(self.arquivo_v0), nome_arquivo[:-4])
        if not os.path.exists(unzip_dir):
            os.mkdir(unzip_dir)

        nomes = self._extrair_nomes(self.fonte_externa.tipo, [fileinfo.filename for fileinfo in zip.infolist()])

        if not nomes:
            raise SincronizadorException(_(u"O arquivo compactado %s não é válido. Confira se os arquivos estão dentro do arquivo compactado.") % self.arquivo_v0)

        try:
            for nome in nomes:

                with zip.open(nome) as src:
                    with open(os.path.join(unzip_dir, os.path.basename(nome)), "wb") as dest:
                        shutil.copyfileobj(src, dest)

        except Exception as e:
            log.error(e.message)

        self.arquivo_alvo = os.path.join(unzip_dir, self._determina_fonte_local(nomes))

    def _descompactar_tar(self):
        """
        Descompacta um arquivo tar e retorna o arquivo
        principal de fonte de dados.
        """

        nome_arquivo = os.path.split(self.arquivo_v0)[-1]
        tar = tarfile.open(self.arquivo_v0,"r:gz")

        if not tarfile.is_tarfile(self.arquivo_v0):
            raise SincronizadorException(u"Não é possível sincronizar a fonte externa %s pois o arquivo tar %s está corrompido ou danificado." % (self.fonte_externa,self.arquivo_v0))

        unzip_dir = os.path.join(os.path.dirname(self.arquivo_v0), nome_arquivo[:-7])
        if not os.path.exists(unzip_dir):
            os.mkdir(unzip_dir)

        members = tar.getmembers()

        nomes = self._extrair_nomes(self.fonte_externa.tipo, [n.name for n in members])

        if not nomes:
            raise SincronizadorException(_(u"FALHA! Não foi possível determinar os arquivos necessários para importação. Arquivos contidos no container: %s") % [n.name for n in members])

        for nome in nomes:
            tar.extract(nome, path=unzip_dir)

        self.arquivo_alvo = os.path.join(unzip_dir, self._determina_fonte_local(nomes))

    def _descompactar_rar(self):
        """
        Descompacta um arquivo rar e determina o arquivo
        da fonte de dados.
        todo: implementar descompatação rar.
        perguntas: é viável? necessário?
        """
        pass

    def _descompactar(self):
        """
        Descompacta um arquivo compactado e retorna o arquivo principal
        de fonte de dados
        """

        # self.processo.log(u"Descompactando arquivo %s." % self.arquivo_v0)

        if self.arquivo_v0[-3:].upper() == "ZIP":
            return self._descompactar_zip()

        elif self.arquivo_v0.upper()[-6:] == "TAR.GZ":
            return self._descompactar_tar()

        elif self.arquivo_v0.upper()[-3:] == "RAR":
            return self._descompactar_rar()

        else:
            log.error(_(u"FALHA! Não foi possível determinar o formato do descompactador a ser utilizado para o arquivo %s.") % self.arquivo_v0)
            raise SincronizadorException(_(u"FALHA! Não foi possível determinar o formato do descompactador a ser utilizado."))

    def _determina_fonte_local(self, nomes):
        """
        Determina, após a extração, qual é nosso arquivo alvo.
        """

        if self.fonte_externa.tipo == "SHAPEFILE":

            if self.fonte_externa.nome_alvo:

                basenames = [os.path.basename(n) for n in nomes]

                fontes_locais = [n for n in basenames if unicode(n).upper() == self.fonte_externa.nome_alvo.upper()]

                fonte_local = fontes_locais[0]

                return fonte_local

            return [os.path.basename(n) for n in nomes if n.upper().endswith("SHP")][0]

        elif self.fonte_externa.tipo == "GEOJSON":
            return nomes[0]
        else:
            raise SincronizadorException(_(u"Não é possível determinar a fonte local deste arquivo."))

    def _extrair_nomes(self, tipo, arquivos):
        """
        Busca os arquivos que devem ser extraídos do arquivo compactado.
        Este método existe pois buscamos um nome arbitário dentro do shapefile e ao longo do caminho,
        aproveitamos para verificar se os auxiliares realmente estão lá.
        """

        log.info(_(u"Validando shapefile compactado"))

        if tipo == "SHAPEFILE":

            shps = [shp for shp in arquivos if os.path.basename(shp).upper().endswith("SHP")]
            dbfs = [dbf for dbf in arquivos if os.path.basename(dbf).upper().endswith("DBF")]
            shxs = [shx for shx in arquivos if os.path.basename(shx).upper().endswith("SHX")]

            # mapeando o tipo prj também, para obtenção de informaçoes de projecao!
            prjs = [prj for prj in arquivos if os.path.basename(prj).upper().endswith("PRJ")]

            if len(shps) < 0:
                log.error(_(u"FALHA! Não existe nenhum shapefile disponível no arquivo compactado."))
                return False

            if len(shps) > 1 and self.fonte_externa.nome_alvo is None:
                log.error(_(u"FALHA! Existe mais de um shapefile disponível no arquivo compactado."))
                return False

            log.info(_(u"Arquivo SHP presente!"))

            try:

                if self.fonte_externa.nome_alvo is not None:

                    shp = [shp for shp in shps if unicode(os.path.basename(shp).upper()) == self.fonte_externa.nome_alvo.upper()][0]

                else:
                    shp = shps[0]

            except IndexError:
                log.error(_(u"FALHA! O arquivo .shp não está disponível no arquivo."))
                return None

            if shp.replace("shp", "shx") not in shxs:
                log.error(_(u"FALHA! O arquivo .shx do shapefile %s não está disponível no arquivo.") % shp)
                return None

            if shp.replace("shp", "dbf") not in dbfs:
                log.error(_(u"FALHA! O arquivo .shx do shapefile %s não está disponível no arquivo.") % shp)
                return None

            log.info(_(u"Arquivos adicionais .dbf e .shx presentes"))
            log.info(_(u"SUCESSO! Arquivo %s contém todas os componentes.") % self.arquivo_v0)

            nomes = []

            if len(prjs) > 0:
                nomes.append(prjs[0])
            return nomes + [shp,
                            shp.replace("shp", "shx"),
                            shp.replace("shp", "dbf")]

        elif tipo == "GEOJSON":

            jsons = [json for json in arquivos if json.upper().endswith("JSON")]

            if len(jsons) <= 0:
                log.error(_(u"FALHA! Não existe nenhum arquivo json disponível no arquivo compactado."))
                return None

            if len(jsons) > 1:
                log.error(_(u"FALHA! Existe mais de um JSON disponível no arquivo compactado."))
                return None

            return jsons[0]

        else:
            msg = _(u"FALHA! Não foi possível extrair os nomes deste arquivo pois este tipo (%s) de fonte externa não é suportado.") % self.fonte_externa.tipo
            log.error(msg)
            return None

    def _validar_campos(self):
        """
        Este método valida se os mapeamentos de campos externos/internos está ok.
        """

        erros = []

        datasource = DataSource(self.arquivo_alvo)

        camada_interna = self.fonte_externa.ref_modelo
        campos_internos = {}
        for ci in camada_interna._meta.fields:
            campos_internos[ci.name] = ci

        camada_externa = datasource[0]
        campos_externos = {}
        for i,ce in enumerate(camada_externa.fields):
            name = ce
            type = camada_externa.field_types[i]
            width = camada_externa.field_widths[i]
            precision = camada_externa.field_precisions[i]
            campos_externos[ce] = OGRField(name,type,width,precision)

        for fm in self.fonte_externa.mapeamentos.filter(~Q(tipo_origem="GEO")):

            interno_existe = campos_internos.has_key(fm.campo_destino)
            externo_existe = campos_externos.has_key(fm.campo_origem)

            if not interno_existe:
                erros.append(ErroMapeamento(fm,"NAO_EXISTE",u"O modelo %s não possui o campo %s." % (fm.camada_interna.name,fm.campo_destino)))

            if not externo_existe:
                erros.append(ErroMapeamento(fm,"NAO_EXISTE",u"A fonte externa %s não possui o campo %s." % (camada_externa.name,fm.campo_origem)))

            if interno_existe and externo_existe:
                campo_interno = campos_internos[fm.campo_destino]
                campo_externo = campos_externos[fm.campo_origem]

                if not ConversorTipoCampo.tipo_coincide(campo_externo.type, fm.tipo_origem):
                    msg = _(u"O campo de origem %(campo_origem)s foi mapeado como %(tipo_mapeado)s, mas %(fonte_externa)s ele tem o tipo %(tipo_externo)s.") % {"campo_origem"  : campo_externo.name,
                                                                                                                                                            "tipo_mapeado"  : fm.tipo_origem,
                                                                                                                                                            "fonte_externa" : camada_externa,
                                                                                                                                                            "tipo_externo"  : campo_externo.type.__name__ }

                    erros.append(ErroMapeamento(fm,"TIPO_INCOMPATIVEL",msg))

        return erros

    def _carrega(self):
        try:

            if not self.arquivo_alvo:
                raise SincronizadorException(_(u"Não foi possível determinar quem é o arquivo alvo para sincronização."))

            temp_datasource = DataSource(self.arquivo_alvo)
            temp_layer = temp_datasource[0]
            temp_srs = temp_layer.srs

            erros_validacao = self._validar_campos()

            if len(erros_validacao) > 0:
                raise SincronizadorException(_(u"Não é possível sincronizar com a fonte externa pois existem erros de campos e mapeamentos. Possivelmente a fonte externa mudou.\nErros:\n %s") % erros_validacao)

            if not temp_srs:

                temp_srs = self.fonte_externa.srid

                lm = LayerMapping(self.fonte_externa.ref_modelo,
                                  self.arquivo_alvo,
                                  self.fonte_externa.mapa_campos,
                                  transform=True,
                                  encoding=self.fonte_externa.encoding,
                                  source_srs=temp_srs)
            else:
                lm = LayerMapping(self.fonte_externa.ref_modelo,
                                  self.arquivo_alvo,
                                  self.fonte_externa.mapa_campos,
                                  encoding=self.fonte_externa.encoding,
                                  source_srs=temp_srs)

            lm.save(verbose=True)

        except Exception as ex:
            log.error(_(u"Ocorreu um erro ao tentar importar os dados da fonte externa.\n %s") % ex.message)
            raise SincronizadorException(_(u"Ocorreu um erro ao tentar importar os dados da fonte externa.\n %s") % ex.message)



import os
from django.conf import settings

ONIVORO_DATA_ROOT = os.path.join(settings.BASE_DIR, "..", "..", "onivoro", "downloads")
ONIVORO_MAIL_NOTIFY = True
ONIVORO_MAIL_RECIPIENTS = None
ONIVORO_MAIL_SENDER = None


# coding: utf-8
from optparse import make_option
from django.core.management import BaseCommand
from onivoro.sincronizador import Sincronizador
from onivoro.models import FonteExterna


class Command(BaseCommand):
    option_list = BaseCommand.option_list + (
        make_option('--fonte',
                    action='store',
                    type='string',
                    default='',
                    dest='nome_fonte'),
    )

    help = """Atualiza dados externos"""

    def handle(self, *args, **options):
        nome_fonte = options.get('nome_fonte')

        fonte_list = FonteExterna.objects.all()
        if nome_fonte:
            fonte_list = fonte_list.filter(nome=nome_fonte)

        for fe in fonte_list:
            self.stdout.write(u'Atualizando %s' % fe.nome)
            s = Sincronizador(fe)
            s.sincronizar()
